/**
  *  \file client/scr-ufo.cc
  *
  *  (c) 2001-2010 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/screens.h"
#include "game/global.h"
#include "util/nls.h"
#include "ui/layout.h"
#include "gfx/palette.h"
#include "game/formula.h"
#include "cpluslib/format.h"
#include "ui/util.h"
#include "client/tiles/objheader.h"
#include "client/tiles/ufoinfo.h"

namespace {
    class WUfoInfoWindow : public UIWindow {
        ObjectHolder h;
        GObjectType& type;
     public:
        WUfoInfoWindow(GObjectType& type);
        bool handleEvent(const UIEvent& e, bool second_pass);
    };
}

/***************************** WUfoInfoWindow ****************************/

WUfoInfoWindow::WUfoInfoWindow(GObjectType& type)
    : UIWindow(_("Ufo Information"), BLUE_WINDOW, COLOR_GRAY),
      type(type)
{
    setLayout(UIVBoxLayout::instance5);

    UIGroup& g1 = h.add(new UIGroup(UIHBoxLayout::instance5));

    g1.add(h.add(new UIButton(_("Close"), SDLK_ESCAPE)));
    g1.add(h.add(new UIButton(_("Go To"), SDLK_RETURN)));
    g1.add(h.add(new UISpacer()));

    add(h.add(new WObjectTypeHeaderTile(type)));
    add(h.add(new WUfoInfoTile(type)));
    add(g1);

    pack();
}

bool
WUfoInfoWindow::handleEvent(const UIEvent& e, bool second_pass)
{
    if (e.type == ev_Key) {
        switch (e.key.code) {
         case SDLK_UP:
         case SDLK_PAGEUP:
         case vk_WheelUp:
            type.setCurrent(type.getPreviousWrap(type.getCurrent(), false));
            return true;
         case SDLK_DOWN:
         case SDLK_PAGEDOWN:
         case vk_WheelDown:
            type.setCurrent(type.getNextWrap(type.getCurrent(), false));
            return true;
         case SDLK_RETURN:
            stop(1);
            return true;
         case SDLK_ESCAPE:
            stop(0);
            return true;
        }
    }
    return UIWindow::handleEvent(e, second_pass);
}



WScreenId
doUfoScreen()
{
    if (getDisplayedTurn().getCurrentUniverse().getUfos().getNumUfos() == 0)
        return WScreenId();

    WUfoInfoWindow win(global_ufo_type);
    ui_root->centerWidget(win);
    int result = ui_root->runModal(win);
    if (result == 1) {
        // "Go to"
        return WScreenId(global_ufo_type.getUfo().getPos());
    } else {
        // "Close"
        return WScreenId();
    }
}

/** Find nearest Ufo. Returns 0 if none found, otherwise slot for GUfoType. */
int
findNearestUfo(GPoint pt)
{
    // FIXME: this has implicit knowledge of the Id assignment scheme of GUfoType
    int found = 0;
    long dist2 = 0;
    bool inside = false;
    int slot = 0;
    const GUfoContainer& ufos = getDisplayedTurn().getCurrentUniverse().getUfos();
    for (GUfoContainer::const_iterator i = ufos.begin(); i != ufos.end(); ++i) {
        long ndist2 = distanceSquared(i->getPos(), pt);
        bool ninside = (ndist2 <= square(i->getRadius()));
        ++slot;
        /* Pick this Ufo, if
           - it's the first one
           - it's closer than our previous choice
           - we're inside it but not inside the previous one */
        if (found <= 0
            || ndist2 < dist2
            || (ninside && !inside))
        {
            found  = slot;
            dist2  = ndist2;
            inside = ninside;
        }
    }
    return found;
}
