/**
  *  \file client/tiles/planetecon.cc
  *
  *  (c) 2001-2010 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/tiles/planetecon.h"
#include "game/planet.h"
#include "ui/widgets.h"
#include "util/nls.h"
#include "cpluslib/format.h"
#include "game/host.h"
#include "game/config.h"
#include "ui/util.h"
#include "game/cmds.h"
#include "game/global.h"

/*************************** WPlanetEconomyTile **************************/

/** \class WPlanetEconomyTile
    \brief Planet Economy Tile

    Displays planetary economy and resources.
    <pre>
    Mineral Mines:     XX
    Factories:         XX
    Defense Posts:     XX
    Neutronium:        XX kt
    Tritanium:         XX kt
    Duranium:          XX kt
    Molybdenum:        XX kt
    Supplies:          XX kt
    </pre> */

WPlanetEconomyTile::WPlanetEconomyTile(GObjectType& type)
    : WObjectTypeObserverTile(GfxPoint(315, font_heights[FONT_NORMAL] * 8), type),
      button_g(0)
{
    setLayout(RightJustified, 0, false);
    addControl(GfxRect(0,  30, 20, 20), h.add(new UIButton("B", 'b')));
    addControl(GfxRect(25, 30, 20, 20), h.add(new UIButton("M", 'm')));
    addControl(GfxRect(0,  55, 20, 20), h.add(new UIButton("S", 's')));
    addControl(GfxRect(25, 55, 20, 20), h.add(new UIButton("D", 'd')));
    addControl(GfxRect(25, 90, 20, 20), h.add(new UIButton("C", 'c')));

    if (host.isPHost() && config.CPEnableGive()) {
        /* Give */
        button_g = &h.add(new UIButton("G", 'g'));
        addControl(GfxRect(25, 0, 20, 20), *button_g);
    }
}

void
WPlanetEconomyTile::drawPlanetEconomyTile(UIAbstractTableWriter& w, bool all_cols)
{
    w.newLColumn();
    w.writeFixedCell(UIColor::tc_Static, _("Mineral Mines:"));
    w.writeFixedCell(UIColor::tc_Static, _("Factories:"));
    w.writeFixedCell(UIColor::tc_Static, _("Defense Posts:"));
    w.writeFixedCell(UIColor::tc_Static, _("Neutronium:"));
    w.writeFixedCell(UIColor::tc_Static, _("Tritanium:"));
    w.writeFixedCell(UIColor::tc_Static, _("Duranium:"));
    w.writeFixedCell(UIColor::tc_Static, _("Molybdenum:"));
    w.writeFixedCell(UIColor::tc_Static, _("Supplies:"));
    if (!all_cols)
        return;

    if (GPlanet* p = dynamic_cast<GPlanet*>(getObject())) {
        w.newRColumn(6*8);
        w.writeVariableCell(UIColor::tc_Green, itoa(p->getStructures(str_Mines)));
        w.writeVariableCell(UIColor::tc_Green, itoa(p->getStructures(str_Factories)));
        w.writeVariableCell(UIColor::tc_Green, itoa(p->getStructures(str_Defense)));
        w.writeVariableCell(UIColor::tc_Green, itoa(p->getCargoRaw(el_Neutronium)));
        w.writeVariableCell(UIColor::tc_Green, itoa(p->getCargoRaw(el_Tritanium)));
        w.writeVariableCell(UIColor::tc_Green, itoa(p->getCargoRaw(el_Duranium)));
        w.writeVariableCell(UIColor::tc_Green, itoa(p->getCargoRaw(el_Molybdenum)));
        w.writeVariableCell(UIColor::tc_Green, itoa(p->getCargoRaw(el_Supplies)));
    
        w.newLColumn();
        w.writeFixedCell(UIColor::tc_Green, "");
        w.writeFixedCell(UIColor::tc_Green, "");
        w.writeFixedCell(UIColor::tc_Green, "");
        w.writeFixedCell(UIColor::tc_Green, "kt");
        w.writeFixedCell(UIColor::tc_Green, "kt");
        w.writeFixedCell(UIColor::tc_Green, "kt");
        w.writeFixedCell(UIColor::tc_Green, "kt");
        w.writeFixedCell(UIColor::tc_Green, "kt");
    }

    w.finish();
}

void
WPlanetEconomyTile::drawData(GfxCanvas& can)
{
    UITableWriter tw(can, getSkin(), getInnerSize(5, 0), FONT_NORMAL);
    drawPlanetEconomyTile(tw, true);

    /* "give" button */
    if (button_g != 0) {
        TFrameColor give_color = cf_None;
        if (GPlanet* p = dynamic_cast<GPlanet*>(getObject())) {
            GCommandContainer& con = getDisplayedTurn().getCommands(p->getOwner());
            if (con.getCommand(GCommand::phc_GivePlanet, p->getId()) != 0)
                give_color = cf_Yellow;
        }
        drawColorFrame(can, button_g->getExtent(), give_color, true);
    }
}
