/**
  *  \file client/tiles/shipmission.cc
  *
  *  (c) 2001-2010 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/tiles/shipmission.h"
#include "ui/uiglobal.h"
#include "game/mission.h"
#include "util/nls.h"
#include "cpluslib/format.h"
#include "game/ship.h"
#include "game/planet.h"
#include "ui/widgets.h"
#include "game/config.h"
#include "game/univ.h"
#include "int/intglobal.h"
#include "int/value.h"
#include "int/data.h"
#include "game/playerset.h"
#include "game/racename.h"

/**************************** WShipMissionTile ***************************/

WShipMissionTile::WShipMissionTile(GObjectType& type)
    : WObjectTypeObserverTile(GfxPoint(315, 9*font_heights[FONT_NORMAL]/2), type)
{
    setLayout(RightJustified, 0, false);
    addControl(GfxRect(27, 0, 20, 20),  h.add(new UIButton("M", 'm')));
    addControl(GfxRect(27, 25, 20, 20), h.add(new UIButton("E", 'e')));
    addControl(GfxRect(27, 50, 20, 20), h.add(new UIButton("F", 'f')));

    cf.add(h.add(new UIButton(cm_Ship_BaseMenu, "B", 'b')).addCommand());
    addControl(GfxRect(0, 48, 24, 24), cf);

    addControl(GfxRect(17, 75, 30, 20), h.add(new UIButton("F9", SDLK_F9)));
}

void
WShipMissionTile::drawData(GfxCanvas& can)
{
    const int y = font_heights[FONT_NORMAL];
    const int x = 120;

    setColor(UIColor::tc_Static);
    outText(can, FONT_NORMAL, 5,   0, _("Mission:"));
    outText(can, FONT_NORMAL, 5,   y, _("Primary Enemy:"));
    outText(can, FONT_NORMAL, 5, 2*y, _("Friendly Code:"));

    // cannot be const.
    GShip* s = dynamic_cast<GShip*>(getObject());
    if (!s)
        return;
    GShip& ship = *s;

    const GMission* msn =
        GMissionList::getInstance().getMissionByNumber(ship.getMission(),
                                                       GPlayerSet(raceId(ship.getRealOwner())));

    setColor(UIColor::tc_Green);
    if (msn)
        outTextF(can, FONT_NORMAL, x-50, 0, 214,
                 msn->getName(ship));
    else
        outTextF(can, FONT_NORMAL, x-50, 0, 214,
                 format("M%d I%d T%d", ship.getMission(), ship.getIntId(), ship.getTowId()));

    outTextF(can, FONT_NORMAL, x,    y, 164-25,
             (ship.getPE()
              ? player_racenames.getShortName(ship.getPE())
              : string_t(_("none"))));
    outTextF(can, FONT_NORMAL, x,  2*y, 100, ship.getFCode());

    // Set color of the 'b' button
    WColorFrame::Color theColor = WColorFrame::cNone;
    if (GUniverse* univ = getType().getUniverse()) {
        int baseId = ship.getOrbit();
        if (baseId != 0 && univ->isValidPlanetId(baseId)) {
            GPlanet& planet = univ->getPlanet(baseId);
            if (planet.isPlayed() && planet.hasBase()) {
                // Planet has a base
                if (planet.getBaseShipyardId() == ship.getId()) {
                    if (planet.getBaseShipyardOrder() == sya_Recycle)
                        theColor = WColorFrame::cRed;
                    else if (planet.getBaseShipyardOrder() == sya_Fix)
                        theColor = WColorFrame::cGreen;
                }
                // FIXME: handle cloning. PCC 1.x uses green if cloning, yellow if conflict
                // (multiple cloners)
            }
        }
    }
    cf.setColor(theColor);

    string_t comment;
    if (IntValue* v = getShipProperties(ship.getId())[sp_Comment])
        comment = v->toString(false);
    if (comment.empty()) {
        setColor(UIColor::tc_Static);
        comment = _("(press [F9] to add a note)");
    } else {
        setColor(UIColor::tc_Yellow);
    }
    outTextF(can, FONT_NORMAL, 5,  7*y/2, 265, comment);
}
